'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var tslib_1 = require('tslib');
var debe = require('debe');

var create = require('asyngular-client').create;
var allowedMethods = [
    debe.types.INSERT,
    debe.types.REMOVE,
    debe.types.GET,
    debe.types.ALL,
    debe.types.COUNT
];
var SocketDebe = (function (_super) {
    tslib_1.__extends(SocketDebe, _super);
    function SocketDebe(_a) {
        var _b = tslib_1.__read(_a, 2), hostname = _b[0], _c = _b[1], port = _c === void 0 ? 8000 : _c;
        return _super.call(this, new SocketAdapter(hostname, port)) || this;
    }
    return SocketDebe;
}(debe.Debe));
var SocketAdapter = (function (_super) {
    tslib_1.__extends(SocketAdapter, _super);
    function SocketAdapter(hostname, port, secure) {
        if (port === void 0) { port = 8000; }
        var _this = _super.call(this) || this;
        _this.socket = create({
            hostname: hostname,
            secure: secure === true || secure === false ? secure : port === 443,
            port: port
        });
        return _this;
    }
    SocketAdapter.prototype.initialize = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                this.start();
                return [2];
            });
        });
    };
    SocketAdapter.prototype.close = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var promise;
            return tslib_1.__generator(this, function (_a) {
                promise = Promise.race([
                    this.socket.listener('disconnect')['once'](),
                    this.socket.listener('connectAbort')['once']()
                ]);
                this.socket.disconnect();
                return [2, promise];
            });
        });
    };
    SocketAdapter.prototype.start = function () {
        var e_1, _a;
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _b, _c, event_1, e_1_1;
            return tslib_1.__generator(this, function (_d) {
                switch (_d.label) {
                    case 0:
                        _d.trys.push([0, 5, 6, 11]);
                        _b = tslib_1.__asyncValues(this.socket.listener('connect'));
                        _d.label = 1;
                    case 1: return [4, _b.next()];
                    case 2:
                        if (!(_c = _d.sent(), !_c.done)) return [3, 4];
                        event_1 = _c.value;
                        _d.label = 3;
                    case 3: return [3, 1];
                    case 4: return [3, 11];
                    case 5:
                        e_1_1 = _d.sent();
                        e_1 = { error: e_1_1 };
                        return [3, 11];
                    case 6:
                        _d.trys.push([6, , 9, 10]);
                        if (!(_c && !_c.done && (_a = _b.return))) return [3, 8];
                        return [4, _a.call(_b)];
                    case 7:
                        _d.sent();
                        _d.label = 8;
                    case 8: return [3, 10];
                    case 9:
                        if (e_1) throw e_1.error;
                        return [7];
                    case 10: return [7];
                    case 11: return [2];
                }
            });
        });
    };
    SocketAdapter.prototype.run = function (action, collection, payload, options) {
        var _this = this;
        if (action === 'insert') {
            return debe.chunkWork(payload, [250000, 10000], function (items) { return _this.socket.invoke('insert', [collection, items, options]); });
        }
        return this.socket.invoke(action, [collection, payload, options]);
    };
    SocketAdapter.prototype.listen = function (action, callback, options) {
        var _this = this;
        var channelId = "" + Math.random()
            .toString(36)
            .substr(2, 9);
        var listen = function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var _a, _b, data, e_2_1;
            var e_2, _c;
            return tslib_1.__generator(this, function (_d) {
                switch (_d.label) {
                    case 0:
                        _d.trys.push([0, 5, 6, 11]);
                        _a = tslib_1.__asyncValues(this.socket.receiver(channelId));
                        _d.label = 1;
                    case 1: return [4, _a.next()];
                    case 2:
                        if (!(_b = _d.sent(), !_b.done)) return [3, 4];
                        data = _b.value;
                        callback(data[0], data[1]);
                        _d.label = 3;
                    case 3: return [3, 1];
                    case 4: return [3, 11];
                    case 5:
                        e_2_1 = _d.sent();
                        e_2 = { error: e_2_1 };
                        return [3, 11];
                    case 6:
                        _d.trys.push([6, , 9, 10]);
                        if (!(_b && !_b.done && (_c = _a.return))) return [3, 8];
                        return [4, _c.call(_a)];
                    case 7:
                        _d.sent();
                        _d.label = 8;
                    case 8: return [3, 10];
                    case 9:
                        if (e_2) throw e_2.error;
                        return [7];
                    case 10: return [7];
                    case 11: return [2];
                }
            });
        }); };
        listen();
        this.socket.transmit('subscribe', [channelId, action, options]);
        var close = function () {
            setTimeout(function () { return _this.socket.closeReceiver(channelId); });
        };
        return close;
    };
    return SocketAdapter;
}(debe.DebeDispatcher));

exports.SocketAdapter = SocketAdapter;
exports.SocketDebe = SocketDebe;
exports.allowedMethods = allowedMethods;
exports.create = create;
